package me.km;

import java.awt.Color;
import java.util.List;
import me.km.blocks.ModBlocks;
import me.km.blocks.cookingpot.TileEntityCookingPot;
import me.km.items.ItemColoredSoup;
import me.km.items.ItemGemStone;
import me.km.items.ModItems;
import me.km.items.ModelCylinder;
import me.km.items.ModelHat;
import me.km.networking.ItemStackDisplayGui;
import me.km.networking.KeyManager;
import me.km.networking.PlayerDisplayGui;
import me.km.networking.PlayerHeadGui;
import me.km.networking.StatusDisplayGui;
import me.km.utils.ClientReflectionUtils;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.screen.MainMenuScreen;
import net.minecraft.client.gui.screen.MultiplayerScreen;
import net.minecraft.client.gui.screen.Screen;
import net.minecraft.client.renderer.color.BlockColors;
import net.minecraft.client.renderer.color.IBlockColor;
import net.minecraft.client.renderer.color.ItemColors;
import net.minecraft.item.Item;
import net.minecraft.item.ArmorItem;
import net.minecraft.item.BlockItem;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.CompoundNBT;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.util.text.StringTextComponent;
import net.minecraft.util.text.TextFormatting;
import net.minecraft.util.text.translation.LanguageMap;
import net.minecraft.world.GrassColors;
import net.minecraft.world.biome.BiomeColors;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.api.distmarker.OnlyIn;
import net.minecraftforge.client.event.GuiScreenEvent;
import net.minecraftforge.client.event.RenderGameOverlayEvent;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.entity.player.ItemTooltipEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.event.TickEvent;

@OnlyIn(Dist.CLIENT)
public class Client 
{   
    public final static ModelCylinder CYLINDER = new ModelCylinder(1.0f);
    public final static ModelHat STRAW_HAT = new ModelHat(1.0f);
    
    public static void init() 
    {
        MinecraftForge.EVENT_BUS.register(new Client());
        MinecraftForge.EVENT_BUS.register(new KeyManager());
        registerColors();
        ClientReflectionUtils.updatePlayerRenderer();
        ClientReflectionUtils.updateFluidRenderer();
    }
    
    private static void registerColors()
    {
        BlockColors bColors = Minecraft.getInstance().getBlockColors();
        bColors.register((state, w, pos, tintIndex) -> 
        {
            if(w == null || pos == null)
            {
                return -16777216;
            }
            TileEntityCookingPot tile = (TileEntityCookingPot) w.getTileEntity(pos);
            if(tile == null) 
            {
                return -16777216;
            }
            Color c = tile.getColor();
            return -16777216 | c.getRed() << 16 | c.getGreen() << 8 | c.getBlue();
        }, ModBlocks.cookingPotCopper);

        IBlockColor tallGrass = (state, w, pos, tintIndex) ->
        {
            if(w != null && pos != null)
            {
                return BiomeColors.getGrassColor(w, pos);
            }
            return GrassColors.get(0.5D, 1.0D);
        };
        bColors.register(tallGrass, ModBlocks.tallGrass);

        ItemColors iColors = Minecraft.getInstance().getItemColors();
        iColors.register((stack, tintIndex) -> tintIndex > 0 ? -1 : ItemColoredSoup.getColor(stack), ModItems.coloredSoup);
        iColors.register((stack, tintIndex) -> tintIndex > 0 ? -1 : ((ItemGemStone) stack.getItem()).getColor(stack), 
                ModItems.gemStone, ModItems.rawGemStone, ModItems.flawlessGemStone);
        
        iColors.register((stack, tintIndex) -> tallGrass.getColor(
                ((BlockItem) stack.getItem()).getBlock().getDefaultState(), 
                null, null, tintIndex), ModBlocks.tallGrass);
    }
    
    @SubscribeEvent
    public void onClientTick(TickEvent.ClientTickEvent e)
    {
        if(e.phase == TickEvent.Phase.END)
        {
            StatusDisplayGui.INSTANCE.tick();
        }  
    }
    
    @SubscribeEvent
    public void onRenderGui(RenderGameOverlayEvent.Pre e)
    {
        if(e.getType() == RenderGameOverlayEvent.ElementType.POTION_ICONS)
        {
            e.setCanceled(true);
        }
    }
    
    @SubscribeEvent
    public void onRenderGui(RenderGameOverlayEvent.Post e)
    {
        if(e.getType() == RenderGameOverlayEvent.ElementType.EXPERIENCE)
        {
            PlayerDisplayGui.INSTANCE.paint();
            StatusDisplayGui.INSTANCE.paint();
            PlayerHeadGui.INSTANCE.paint();
            ItemStackDisplayGui.INSTANCE.paint();
        }
    }

    @SubscribeEvent
    public void onRenderGui(GuiScreenEvent.InitGuiEvent e)
    {
        // workaround until a client side server leave event is back
        Screen gui = e.getGui();
        if(gui instanceof MainMenuScreen || gui instanceof MultiplayerScreen)
        {
            PlayerDisplayGui.INSTANCE.clear();
            StatusDisplayGui.INSTANCE.clear();
            PlayerHeadGui.INSTANCE.clear();
            ItemStackDisplayGui.INSTANCE.clear();
            ItemStackDisplayGui.INSTANCE.setActive(false);
        }
    }
    
    @SubscribeEvent
    public void showingTooltip(ItemTooltipEvent e) 
    {
        ItemStack stack = e.getItemStack();
        Item item = stack.getItem();
        if(item instanceof ArmorItem)
        {
            CompoundNBT com = stack.getTag();
            if(com != null && com.contains("magic"))
            {
                int magic = com.getInt("magic");
                if(magic != 0)
                {
                    List<ITextComponent> list = e.getToolTip();
                    for(int i = 0; i < list.size(); i++)
                    {
                        if(list.get(i).getString().contains("+"))
                        {
                            list.add(i, new StringTextComponent(
                                    String.format("%s+%d %s", TextFormatting.BLUE, magic, 
                                            LanguageMap.getInstance().translateKey("magicarmor"))));
                            break;
                        }
                    }
                }
            }
        }
    }
    
    /*private static int[] counter = new int[5];
    private static final double BASE_ARRAY_SIZE = 16 * 256 * 16 * 16 + 32;
    
    @SubscribeEvent
    public void test(net.minecraftforge.client.event.ClientChatEvent e)
    {
        net.minecraft.world.chunk.Chunk c = Minecraft.getInstance().player.world.getChunkAt(Minecraft.getInstance().player.getPosition());
        
        if(e.getMessage().equals("run"))
        {
            countRun(Minecraft.getInstance().player.world, 
                    c.getPos().getXStart(), c.getPos().getZStart(), 
                    c.getPos().getXStart() + 16, c.getPos().getZStart() + 16);
            return;
        }
        if(!e.getMessage().equals("octree"))
        {
            return;
        }
        
        counter[0] = 0; // 16
        counter[1] = 0; // 8
        counter[2] = 0; // 4
        counter[3] = 0; // 2
        counter[4] = 0; // 1
        
        for(int i = 0; i < 256; i += 16)
        {
            count(Minecraft.getInstance().player.world, 
                    c.getPos().getXStart(), i, c.getPos().getZStart(), 
                    c.getPos().getXStart() + 16, i + 16, c.getPos().getZStart() + 16, 
                    0);
        }
        
        System.out.println("16: " + counter[0]);
        System.out.println("8: " + counter[1]);
        System.out.println("4: " + counter[2]);
        System.out.println("2: " + counter[3]);
        System.out.println("1: " + counter[4]);
        
        int size = 16 + 32 * 16;
        int a = 16 - counter[0];
        size += a * 264;
        
        a *= 8;
        a -= counter[1];
        size += a * 264;
        
        a *= 8;
        a -= counter[2];
        size += a * 264;
        
        a *= 8;
        a -= counter[3];
        size += a * 128;
        
        a *= 8;
        a -= counter[4];
        
        System.out.println("Size: " + size + " Bit check " + a);
        System.out.println("Compression: " + (BASE_ARRAY_SIZE / size));
    }
    
    private static void countRun(net.minecraft.world.World w, int minX, int minZ, int maxX, int maxZ)
    {
        int groups = 0;
        for(int x = minX; x < maxX; x++)
        {
            for(int z = minZ; z < maxZ; z++)
            {
                groups += countTower(w, x, z);
            }
        }
        
        int size = groups * (16 + 8 + 32);
        int doubleSize = groups * (16 + 8 + 32 * 2);
        
        System.out.println("Size 1: " + size);
        System.out.println("Size 2: " + doubleSize);
        System.out.println("Compression 1: " + (BASE_ARRAY_SIZE / size));
        System.out.println("Compression 2: " + (BASE_ARRAY_SIZE / doubleSize));
    }
    
    private static int countTower(net.minecraft.world.World w, int x, int z)
    {
        net.minecraft.util.math.BlockPos.MutableBlockPos pos = new net.minecraft.util.math.BlockPos.MutableBlockPos();
        pos.setPos(x, 255, z);
        net.minecraft.block.Block b = w.getBlockState(pos).getBlock();
        int groups = 1;
        for(int y = 255; y >= 0; y--)
        {
            pos.setPos(x, y, z);
            net.minecraft.block.Block ob = w.getBlockState(pos).getBlock();
            if(b != ob)
            {
                groups++;
                b = ob;
            }
        }
        return groups;
    }
    
    private static void count(net.minecraft.world.World w, int minX, int minY, int minZ, int maxX, int maxY, int maxZ, int depth)
    {
        if(isSameBlock(w, minX, minY, minZ, maxX, maxY, maxZ))
        {
            counter[depth]++;
            return;
        }
        if(depth == 4)
        {
            counter[4]++;
            return;
        }
        
        int midX = (minX + maxX) >> 1;
        int midY = (minY + maxY) >> 1;
        int midZ = (minZ + maxZ) >> 1;
        
        depth++;
        count(w, minX, minY, minZ, midX, midY, midZ, depth);
        count(w, midX, minY, minZ, maxX, midY, midZ, depth);
        count(w, minX, minY, midZ, midX, midY, maxZ, depth);
        count(w, midX, minY, midZ, maxX, midY, maxZ, depth);
        
        count(w, minX, midY, minZ, midX, maxY, midZ, depth);
        count(w, midX, midY, minZ, maxX, maxY, midZ, depth);
        count(w, minX, midY, midZ, midX, maxY, maxZ, depth);
        count(w, midX, midY, midZ, maxX, maxY, maxZ, depth);
    }
    
    private static boolean isSameBlock(net.minecraft.world.World w, int minX, int minY, int minZ, int maxX, int maxY, int maxZ)
    {
        net.minecraft.util.math.BlockPos.MutableBlockPos pos = new net.minecraft.util.math.BlockPos.MutableBlockPos();
        pos.setPos(minX, minY, minZ);
        net.minecraft.block.Block b = w.getBlockState(pos).getBlock();
        for(int x = minX; x < maxX; x++)
        {
            for(int y = minY; y < maxY; y++)
            {
                for(int z = minZ; z < maxZ; z++)
                {
                    pos.setPos(x, y, z);
                    if(b != w.getBlockState(pos).getBlock())
                    {
                        return false;
                    }
                }
            }
        }
        return true;
    }*/
}
